"""
Author: DiChen
Date: 2024-09-08 11:46:48
LastEditors: DiChen
LastEditTime: 2024-09-09 21:59:50
"""

"""
Author: DiChen
Date: 2024-09-08 11:46:48
LastEditors: DiChen
LastEditTime: 2024-09-08 11:46:51
"""

# exceptions.py


# 基础自定义异常类
class MyBaseError(Exception):
    """自定义基础异常类"""

    def __init__(self, message, error_code=None):
        super().__init__(message)
        self.error_code = error_code

    def __str__(self):
        if self.error_code:
            return f"[Error {self.error_code}]: {self.args[0]}"
        else:
            return self.args[0]


# 具体的自定义异常类
class NotValueError(MyBaseError):
    """data is none"""

    def __init__(self, message="not Value", error_code=1001):
        super().__init__(message, error_code)


class DatabaseError(MyBaseError):
    """database error"""

    def __init__(self, message="database error", error_code=1002):
        super().__init__(message, error_code)


class NetworkError(MyBaseError):
    """network error"""

    def __init__(self, message="network error", error_code=1003):
        super().__init__(message, error_code)


class modelStatusError(MyBaseError):
    """model status error"""

    def __init__(self, message="model status error", error_code=1004):
        super().__init__(message, error_code)


class calTimeoutError(MyBaseError):
    """model calculate timeout error"""

    def __init__(self, message="model calculate timeout error", error_code=1005):
        super().__init__(message, error_code)


class UploadFileError(MyBaseError):
    """upload file error"""

    def __init__(self, message="upload file error", error_code=1006):
        super().__init__(message, error_code)


class MDLVaildParamsError(MyBaseError):
    """MDL vaild params error"""

    def __init__(self, message="MDL vaild params error", error_code=1007):
        super().__init__(message, error_code)



"""
Author: DiChen
Date: 2024-09-06 15:22:20
LastEditors: DiChen
LastEditTime: 2024-09-11 18:37:16
"""

import sys
from . import constants as C
from .openUtils.http_client import HttpClient


class Service:
    def __init__(self, token: str = None):
        # res = HttpClient.hander_response(
        #     HttpClient.get_sync(
        #         url="http://www.baidu.com"
        #     )
        # )
        self.token: str = token
        self.portalUrl = C.basePortalUrl
        self.managerUrl = C.baseManagerUrl
        self.dataUrl = C.baseDataUrl
        if not (self.portalUrl or self.managerUrl or self.dataUrl):
            print("读取配置文件有误，请联系管理员！")
            sys.exit(1)
        self._validateToken()

    #########################private#########################################
    def _validateToken(self):
        res = HttpClient.hander_response(
            HttpClient.get_sync(
                self.portalUrl + C.CHECK_SDK, params={"token": self.token}
            )
        ).get("json", {})
        if res.get("data") != 1:
            print("token无效，请联系管理员！")
            sys.exit(1)



import requests
import tenacity
from openai import OpenAI


class AcademicQueryService:
    def __init__(self, api_key: str = "sk-4bp5a1DcdLSHCiw1401270055f47424b9eA58cAd587266A3",
                 modelingContext: str = "",
                 base_url: str = "https://aihubmix.com/v1"):
        self.client = OpenAI(api_key=api_key, base_url=base_url)
        self.modelingContext = modelingContext

    async def get_academic_question_answer(self, query: str) -> dict:
        """
        获取学术问题的答案（这是对外暴露的主要接口）
        """
        paper_list = await self._get_academic_question_answer_list(query)
        question_answer = await self._get_openai_summary(query, paper_list)
        return {
            "question": query,
            "answer": question_answer,
            "paperList": paper_list
        }

    @tenacity.retry(wait=tenacity.wait_fixed(1), stop=tenacity.stop_after_attempt(5))
    async def _get_academic_question_answer_list(self, query: str, page: int = 1, size: int = 10) -> list:
        """内部方法：获取论文列表 - 使用新的两步调用流程"""
        try:
            # 第一步：创建搜索线程，获取 search_id
            search_url = "https://consensus.app/api/threads/"
            search_payload = {
                "user_message": query,
                "is_pro_enabled": False,
                "is_incognito": False,
                "size": size,
                "filters": {},
                "search_mode": "SUMMARY"
            }
            search_headers = {
                'Referer': 'https://consensus.app',
                'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36',
                'Content-Type': 'application/json',
                'Accept': 'application/json, text/plain, */*'
            }

            search_response = requests.post(
                search_url, json=search_payload, headers=search_headers, timeout=15)

            if search_response.status_code not in [200, 201]:
                # 降级到直接查询方式
                return await self._fallback_direct_query(query, page, size)

            search_data = search_response.json()
            search_id = None

            # 尝试从不同位置获取search_id
            if 'search_id' in search_data:
                search_id = search_data['search_id']
            elif 'interactions' in search_data and len(search_data['interactions']) > 0:
                first_interaction = search_data['interactions'][0]
                search_id = first_interaction.get('search_id')

            if not search_id:
                return await self._fallback_direct_query(query, page, size)

            # 第二步：使用 search_id 获取论文结果
            papers_url = f"https://consensus.app/api/paper_search/{search_id}/?page={page-1}&size={size}"
            papers_headers = {
                'Referer': 'https://consensus.app',
                'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36',
                'Accept': 'application/json, text/plain, */*'
            }

            papers_response = requests.get(
                papers_url, headers=papers_headers, timeout=15)

            if papers_response.status_code != 200:
                return await self._fallback_direct_query(query, page, size)

            papers_data = papers_response.json()
            papers = papers_data.get("papers", [])

            return papers[:size]

        except Exception as e:
            # 降级到直接查询方式
            return await self._fallback_direct_query(query, page, size)

    async def _fallback_direct_query(self, query: str, page: int = 1, size: int = 10) -> list:
        """降级方案：直接查询论文（原来的方法）"""
        try:
            url = "https://consensus.app/api/paper_search/?query=" + \
                query + "&page=" + str(page) + "&size=" + str(size)
            payload = {}
            headers = {
                'Referer': 'https://consensus.app',
                'User-Agent': 'Apifox/1.0.0 (https://apifox.com)'
            }
            response = requests.request(
                "GET", url, headers=headers, data=payload).json()

            # return top 10 result in claims
            top10Claims = response["papers"][:10]
            return top10Claims
        except Exception as e:
            return []

    async def _get_openai_summary(self, query: str, paper_list: list) -> str:
        """内部方法：获取 OpenAI 总结"""
        paperContext = ""
        for paper in paper_list:
            # 组织成，这是一篇名为<title>的论文，发表在<journal>上，相关的内容为<display_text>。
            paperContext += "This is a paper named: " + \
                paper["title"] + " published in: " + paper["journal"] + \
                ". The related content is: " + paper["display_text"] + "."

        completion = self.client.chat.completions.create(
            model="gpt-4.1-nano",
            messages=[
                {"role": "system", "content": "You are a helpful AI researcher assistant. User is now conducting scientific geoscience research. The modeling history is: " + self.modelingContext +
                    ". He/she is now asking a question about " + query + ".  And here are some information related to this question: " + paperContext + ". And your task is to summarize the information and answer the question. Please answer the question in English."},
                {
                    "role": "user",
                    "content": "Besides your summary, you should not reply any other information. Below is your answer to the question based on the information provided: "
                }
            ]
        )
        return completion.choices[0].message.content



import httpx
from typing import Optional, Dict, Any, Union, Tuple
import asyncio

# 类型别名
Headers = Dict[str, str]
Files = Dict[str, Tuple[str, Any]]


class HttpClient:
    @staticmethod
    def _make_sync_request(
        method: str,
        url: str,
        timeout: int = 10,
        data: Optional[Union[Dict[str, Any], str, bytes]] = None,
        json: Optional[Dict[str, Any]] = None,
        files: Optional[Files] = None,
        params: Optional[Dict[str, Any]] = None,
        headers: Optional[Headers] = None,
        downloadFile: bool = False,
    ) -> Dict[str, Any]:
        try:
            with httpx.Client(timeout=httpx.Timeout(timeout)) as client:
                response = client.request(
                    method,
                    url,
                    data=data,
                    json=json,
                    files=files,
                    params=params,
                    headers=headers,
                )
                print
                response.raise_for_status()  # 检查 HTTP 错误
                if downloadFile:
                    return {
                        "status_code": response.status_code,
                        "headers": dict(response.headers),
                        "content": response.content,
                    }
                return {
                    "status_code": response.status_code,
                    "headers": dict(response.headers),
                    "json": response.json(),
                }
        except httpx.TimeoutException:
            # 处理超时错误
            return {"status_code": None, "headers": None, "error": "Request timed out"}
        except httpx.HTTPStatusError as e:
            return {
                "status_code": e.response.status_code,
                "headers": dict(e.response.headers),
                "error": str(e),
            }
        except httpx.RequestError as e:
            return {
                "status_code": None,
                "headers": None,
                "error": f"Request error: {e}",
            }
        except Exception as e:
            return {
                "status_code": None,
                "headers": None,
                "error": f"Unexpected error: {e}",
            }

    @staticmethod
    def hander_response(response):
        if response["status_code"] == 200:
            return response
        else:
            raise Exception(response["error"])

    @staticmethod
    def get_sync(
        url: str,
        timeout: int = 10,
        params: Optional[Dict[str, Any]] = None,
        headers: Optional[Headers] = None,
    ) -> Dict[str, Any]:
        return HttpClient._make_sync_request(
            "GET", url, timeout=timeout, params=params, headers=headers
        )

    @staticmethod
    def get_file_sync(
        url: str,
        timeout: int = 10,
        params: Optional[Dict[str, Any]] = None,
        headers: Optional[Headers] = None,
    ):
        response = HttpClient._make_sync_request(
            "GET",
            url,
            timeout=timeout,
            params=params,
            headers=headers,
            downloadFile=True,
        )
        return response

    @staticmethod
    def post_sync(
        url: str,
        timeout: int = 10,
        data: Optional[Union[Dict[str, Any], str, bytes]] = None,
        json: Optional[Dict[str, Any]] = None,
        files: Optional[Files] = None,
        headers: Optional[Headers] = None,
    ) -> Dict[str, Any]:
        return HttpClient._make_sync_request(
            "POST",
            url,
            timeout=timeout,
            data=data,
            json=json,
            files=files,
            headers=headers,
        )

    @staticmethod
    def put_sync(
        url: str,
        timeout: int = 10,
        data: Optional[Union[Dict[str, Any], str, bytes]] = None,
        json: Optional[Dict[str, Any]] = None,
        headers: Optional[Headers] = None,
    ) -> Dict[str, Any]:
        return HttpClient._make_sync_request(
            "PUT", url, timeout=timeout, data=data, json=json, headers=headers
        )

    @staticmethod
    def delete_sync(
        url: str,
        timeout: int = 10,
        params: Optional[Dict[str, Any]] = None,
        headers: Optional[Headers] = None,
    ) -> Dict[str, Any]:
        return HttpClient._make_sync_request(
            "DELETE", url, timeout=timeout, params=params, headers=headers
        )

    @staticmethod
    async def _make_async_request(
        method: str,
        url: str,
        timeout: int = 10,
        data: Optional[Union[Dict[str, Any], str, bytes]] = None,
        json: Optional[Dict[str, Any]] = None,
        files: Optional[Files] = None,
        params: Optional[Dict[str, Any]] = None,
        headers: Optional[Headers] = None,
    ) -> Dict[str, Any]:
        try:
            async with httpx.AsyncClient(timeout=httpx.Timeout(timeout)) as client:
                response = await client.request(
                    method,
                    url,
                    data=data,
                    json=json,
                    files=files,
                    params=params,
                    headers=headers,
                )
                response.raise_for_status()  # 检查 HTTP 错误
                return {
                    "status_code": response.status_code,
                    "headers": dict(response.headers),
                    "json": response.json(),
                }
        except httpx.TimeoutException:
            # 处理超时错误
            return {"status_code": None, "headers": None, "error": "Request timed out"}
        except httpx.HTTPStatusError as e:
            return {
                "status_code": e.response.status_code,
                "headers": dict(e.response.headers),
                "error": str(e),
            }
        except httpx.RequestError as e:
            return {
                "status_code": None,
                "headers": None,
                "error": f"Request error: {e}",
            }
        except Exception as e:
            return {
                "status_code": None,
                "headers": None,
                "error": f"Unexpected error: {e}",
            }

    @staticmethod
    async def get_async(
        url: str,
        timeout: int = 10,
        params: Optional[Dict[str, Any]] = None,
        headers: Optional[Headers] = None,
    ) -> Dict[str, Any]:
        return await HttpClient._make_async_request(
            "GET", url, timeout=timeout, params=params, headers=headers
        )

    @staticmethod
    async def post_async(
        url: str,
        timeout: int = 10,
        data: Optional[Union[Dict[str, Any], str, bytes]] = None,
        json: Optional[Dict[str, Any]] = None,
        files: Optional[Files] = None,
        headers: Optional[Headers] = None,
    ) -> Dict[str, Any]:
        return await HttpClient._make_async_request(
            "POST",
            url,
            timeout=timeout,
            data=data,
            json=json,
            files=files,
            headers=headers,
        )

    @staticmethod
    async def put_async(
        url: str,
        timeout: int = 10,
        data: Optional[Union[Dict[str, Any], str, bytes]] = None,
        json: Optional[Dict[str, Any]] = None,
        headers: Optional[Headers] = None,
    ) -> Dict[str, Any]:
        return await HttpClient._make_async_request(
            "PUT", url, timeout=timeout, data=data, json=json, headers=headers
        )

    @staticmethod
    async def delete_async(
        url: str,
        timeout: int = 10,
        params: Optional[Dict[str, Any]] = None,
        headers: Optional[Headers] = None,
    ) -> Dict[str, Any]:
        return await HttpClient._make_async_request(
            "DELETE", url, timeout=timeout, params=params, headers=headers
        )



"""
Author: DiChen
Date: 2024-09-06 15:14:57
LastEditors: DiChen
LastEditTime: 2024-09-07 00:16:30
"""

from . import *
import secrets
import os


class OGMSTask(Service):
    def __init__(self, origin_lists: dict, token: str = None):
        super().__init__(token=token)
        self.status: None | int = None
        self.username = token
        PV.v_empty(origin_lists, "origin lists")
        self.origin_lists = origin_lists
        self.subscirbe_lists = {}
        self.tid = None
        # 对输入的参数进行校验、文件上传等操作

    def wait4Status(self, timeout: int = 7200):
        try:
            start_time = time.time()
            stateManager = StateManager()
            stateManager.checkInputStatus(PV.v_status(self._refresh()))
            while stateManager.hasStatus(0b100) is False:
                stateManager.checkInputStatus(PV.v_status(self._refresh()))
                if time.time() - start_time > timeout:
                    raise calTimeoutError()
                time.sleep(3)
            return {
                "outputs": self.outputs,
            }

        except NotValueError or modelStatusError as e:
            print(e)
            exit(1)

    def configInputData(self, params: dict):
        try:
            PV.v_empty(params, "params list")
            lists = {"inputs": self._uploadData(
                params), "username": self.username}
            return self._mergeData(lists)
        except NotValueError or UploadFileError or MDLVaildParamsError as e:
            print(e)
            exit(1)

    ######################## private################################
    def _uploadData(self, pathList: dict):
        inputs = {}
        for category, files in pathList.items():
            inputs[category] = {}
            for key, value in files.items():
                # 判断是数值参数还是文件参数
                if isinstance(value, (str, int, float)) and not str(value).startswith('/'):
                    # 数值参数：生成XML，上传并返回url；同时保留children用于填充值
                    xml_content = self._create_value_xml(str(key), str(value))
                    xml_url = self._upload_xml_string(
                        xml_content, f"{key}.xml")
                    inputs[category][key] = {
                        "name": f"{key}.xml",
                        "url": xml_url,
                        "children": [{str(key): str(value)}],
                    }
                else:
                    # 文件参数：上传文件并获取URL
                    file_path = str(value)
                    file_name = file_path.split("/")[-1]
                    inputs[category][key] = {
                        "name": file_name,
                        "url": self._getUploadData(file_path),
                    }
        return inputs

    def _getUploadData(self, path: str):
        res = (
            HttpClient.hander_response(
                HttpClient.post_sync(
                    self.dataUrl + C.UPLOAD_DATA, files={"datafile": open(path, "rb")}
                )
            )
            .get("json", {})
            .get("data", {})
        )
        if res.get("id"):
            return self.dataUrl+C.UPLOAD_DATA + res.get(
                "id"
            )
        raise UploadFileError()

    def _create_value_xml(self, name: str, value: str) -> str:
        """根据数值参数生成与服务端兼容的XML内容。"""
        # 参考 testify 目录示例：
        # <Dataset> <XDO name="system_efficiency" kernelType="string" value="0.8" /> </Dataset>
        return f'<Dataset> <XDO name="{name}" kernelType="string" value="{value}" /> </Dataset>'

    def _upload_xml_string(self, xml_content: str, filename: str) -> str:
        """将XML字符串写入临时文件并复用文件上传接口，返回url。"""
        import tempfile
        import os

        with tempfile.NamedTemporaryFile(mode="w", suffix=".xml", delete=False, encoding="utf-8") as tmp_file:
            tmp_file.write(xml_content)
            tmp_path = tmp_file.name
        try:
            return self._getUploadData(tmp_path)
        finally:
            if os.path.exists(tmp_path):
                os.unlink(tmp_path)

    def _mergeData(self, params: dict):
        def extract_file_suffix(filename: str) -> str:
            """提取文件名的后缀名."""
            return filename.split(".")[-1] if "." in filename else ""

        def update_input_item(input_item: dict, event_data: dict):
            """
            根据 input_data 中的 event_data 更新 origin_data 中的 input_item。
            支持处理直接传递的 'value' 字段和children结构。
            """
            # ✅ 处理直接传递value的情况（数值参数）
            if "value" in event_data and "url" not in event_data:
                # 数值参数：如 {"value": "0.8"}
                if "children" in input_item and input_item["children"]:
                    child = input_item["children"][0]
                    child["value"] = event_data["value"]
                    input_item["suffix"] = "xml"  # 数值参数使用xml格式
                return  # 早返回，避免执行后面的逻辑

            # 原有的处理逻辑
            if "children" in event_data:
                input_item["suffix"] = "xml"  # 如果有 children，后缀名固定为 xml
                for child in input_item.get("children", []):
                    event_name = child["eventName"]
                    for b_child in event_data["children"]:
                        if event_name in b_child:
                            child["value"] = b_child[event_name]
            else:
                if "name" in event_data:
                    input_item["suffix"] = extract_file_suffix(
                        event_data["name"])

            if "url" in event_data:
                input_item["url"] = event_data["url"]

        def fill_data_with_input(input_data: dict, origin_data: dict) -> dict:
            """根据 input_data 填补 origin_data."""
            for input_item in origin_data.get("inputs", []):
                state_name = input_item.get("statename")
                event_name = input_item.get("event")

                PV.v_empty(state_name, "State name")
                PV.v_empty(event_name, "Event name")

                state_data = input_data["inputs"].get(state_name)
                if state_data and event_name in state_data:
                    update_input_item(input_item, state_data[event_name])
            origin_data["username"] = input_data.get("username")
            return origin_data

        filled_origin_data = fill_data_with_input(params, self.origin_lists)
        return self._validData(filled_origin_data)

    def _validData(self, merge_data: dict):
        def validate_event(event):
            errors = []
            event_name = f"{event.get('statename')}-{event.get('event')}"

            # 检查是否是数值参数（有children但没有url）
            is_numeric_param = "children" in event and not event.get("url")

            if event.get("optional") == "False":
                # 必填项
                if is_numeric_param:
                    # 数值参数：只检查children和suffix
                    if not event.get("suffix"):
                        errors.append(f"{event_name}的文件格式有误！")
                    for child in event["children"]:
                        if not child.get("value"):
                            errors.append(f"{event_name}子参数有误")
                else:
                    # 文件参数：检查url和suffix
                    if not event.get("url"):
                        errors.append(f"{event_name}的中转数据信息有误！")
                    if not event.get("suffix"):
                        errors.append(f"{event_name}的文件有误！")
            elif event.get("optional") == "True":
                # 选填项
                if event.get("url") or event.get("suffix") or "children" in event:
                    if not (event.get("url") and event.get("suffix")):
                        errors.append(f"{event_name}子参数有误！")
                    if "children" in event:
                        for child in event["children"]:
                            if not child.get("value"):
                                errors.append(f"{event_name}子参数不能为空！")

            return errors

        def process_inputs(inputs):
            errors = []
            valid_inputs = []
            for event in inputs:
                event_errors = validate_event(event)
                if event_errors:
                    errors.extend(event_errors)
                else:
                    if event.get("optional") == "True":
                        if not (
                            event.get("url")
                            or event.get("suffix")
                            or "children" in event
                        ):
                            continue
                    valid_inputs.append(event)
            return valid_inputs, errors

        def check_username(username):
            errors = []
            if not username:
                errors.append("no token")
            return errors

        errors = check_username(merge_data.get("username"))

        # 处理 inputs
        valid_inputs, input_errors = process_inputs(
            merge_data.get("inputs", []))
        errors.extend(input_errors)

        # 更新数据
        merge_data["inputs"] = valid_inputs

        # 打印错误信息
        if errors:
            raise MDLVaildParamsError("\n".join(errors))
        else:
            self.subscirbe_lists = merge_data
            return 1

    def _refresh(self):
        PV.v_empty(self.modelSign, "Model sign")
        res = HttpClient.hander_response(
            HttpClient.post_sync(
                url=self.managerUrl + C.REFRESH_RECORD, json=self.modelSign
            )
        ).get("json", {})
        if res.get("code") == 1:
            if res.get("data").get("status") != 2:
                return res.get("data").get("status")
            else:
                hasValue = False
                for output in res["data"]["outputs"]:
                    if output.get("url") is not None and output.get("url") != "":
                        url = output.get("url")
                        # updated_url = url.replace(
                        #     "http://112.4.132.6:8083",
                        #     "http://geomodeling.njnu.edu.cn/dataTransferServer",
                        # )
                        output["url"] = url
                        hasValue = True
                if hasValue is False:
                    return -1
                for output in res["data"]["outputs"]:
                    if "[" in output.get("url"):
                        output["multiple"] = True
                self.outputs = res["data"]["outputs"]
                return 2
        return -2


class OGMSAccess(Service):
    def __init__(self, modelName: str, token: str = None):
        super().__init__(token=token)
        PV.v_empty(modelName, "Model name")
        self.modelName = modelName
        self.outputs = []
        if self._checkModelService(pid=self._checkModel(modelName=modelName)):
            print("Model service is ready!")
        else:
            print("Model service is not ready, please try again later!")
            exit(1)

    def createTask(self, params: dict):
        PV.v_empty(params, "Params")
        task = OGMSTask(self.originLists, self.token)
        if task.configInputData(params) and self._subscribeTask(task):
            result = task.wait4Status()
            self.outputs = result["outputs"]
            return self.outputs

    def downloadAllData(self):

        s_id = secrets.token_hex(8)
        downloadFilesNum = 0
        downlaodedFilesNum = 0
        if not self.outputs:
            print("没有可下载的数据")
            return False

        for output in self.outputs:
            statename = output["statename"]
            event = output["event"]
            url = output["url"]
            suffix = output["suffix"]
            # 构建文件名
            base_filename = f"{statename}-{event}"
            filename = f"{base_filename}.{suffix}"
            counter = 1

            file_path = "./data/" + self.modelName + "_" + s_id + "/" + filename

            dir_path = os.path.dirname(file_path)
            if not os.path.exists(dir_path):
                os.makedirs(dir_path)

            # 检查文件是否存在
            while os.path.exists(file_path):
                filename = f"{base_filename}_{counter}.{suffix}"
                file_path = "./data/" + self.modelName + "_" + s_id + "/" + filename
                counter += 1
            downloadFilesNum = downloadFilesNum + 1
            # 下载文件并保存
            content = HttpClient.hander_response(HttpClient.get_file_sync(url=url)).get(
                "content", {}
            )
            if content:
                with open(file_path, "wb") as f:
                    f.write(content)
                print(f"Downloaded {filename}")
                downlaodedFilesNum = downlaodedFilesNum + 1
            else:
                print(f"Failed to download {url}")
        if downlaodedFilesNum == 0:
            print("Failed to download files")
            return False
        if downloadFilesNum == downlaodedFilesNum:
            print("All files downloaded successfully")
            return True
        else:
            print("Failed to download some files")
            return True

    ######################## private################################
    def _checkModel(self, modelName: str):
        PV.v_empty(modelName, "Model name")
        res = (
            HttpClient.hander_response(
                HttpClient.get_sync(
                    self.portalUrl + C.CHECK_MODEL +
                    urllib.parse.quote(modelName)
                )
            )
            .get("json", {})
            .get("data", {})
        )
        if res.get("md5"):
            self.originLists = MDL().resolvingMDL(res)
            if self.originLists:
                return res.get("md5")
        return 0

    def _checkModelService(self, pid: str):
        PV.v_empty(pid, "Model pid")
        if (
            HttpClient.hander_response(
                HttpClient.get_sync(
                    self.managerUrl + C.CHECK_MODEL_SERVICE + pid)
            )
            .get("json", {})
            .get("data", {})
            == True
        ):
            return 1
        return 0

    def _subscribeTask(self, task):
        res = HttpClient.hander_response(
            HttpClient.post_sync(
                self.managerUrl + C.INVOKE_MODEL, json=task.subscirbe_lists
            )
        ).get("json", {})
        if res.get("code") == 1:
            task.ip = res.get("data").get("ip")
            task.port = res.get("data").get("port")
            task.tid = res.get("data").get("tid")
            task.modelSign = {"port": task.port,
                              "ip": task.ip, "tid": task.tid}
            return 1
        raise NotValueError("Model invoke error!")



"""
Author: DiChen
Date: 2024-09-10 18:54:44
LastEditors: DiChen
LastEditTime: 2024-09-10 18:54:56
"""

###################### configPath######################
basePortalUrl = "http://222.192.7.75"
# basePortalUrl = "http://172.21.212.251:7777"
baseManagerUrl = "http://222.192.7.75/managerServer"
baseDataUrl = "http://222.192.7.75/dataTransferServer"


###################### apiPath########################
CHECK_MODEL = "/computableModel/ModelInfo_name/"
CHECK_MODEL_SERVICE = "/GeoModeling/task/verify/"
INVOKE_MODEL = "/GeoModeling/computableModel/invoke"
REFRESH_RECORD = "/GeoModeling/computableModel/refreshTaskRecord"
UPLOAD_DATA = "/data/"
CHECK_SDK = "/sdk/check_test/"



"""
Author: DiChen
Date: 2024-09-06 17:21:20
LastEditors: DiChen
LastEditTime: 2024-09-13 21:54:57
"""

"""
Author: DiChen
Date: 2024-09-06 17:21:20
LastEditors: DiChen
LastEditTime: 2024-09-08 16:23:59
"""

# 定义状态常量
STATE_INIT = 0b1  # 1: init
STATE_RUNNING = 0b10  # 2: running
STATE_COMPLETED = 0b100  # 4: completed
STATE_ERROR = 0b1000  # 8: error


class StateManager:
    def __init__(self):
        # init state
        self.state = STATE_INIT
        print(f"StateManager initialized in state: {bin(self.state)}")

    def addState(self, state):
        """add state"""
        self.state |= state

    def removeState(self, state):
        """remove state"""
        self.state &= ~state

    def hasStatus(self, state):
        """check state"""
        return self.state & state != 0

    def trans2Status(self, state):
        """transition to state"""
        if state == STATE_RUNNING and not self.hasStatus(STATE_INIT):
            print("Cannot run without initialization")
            return

        if state == STATE_COMPLETED and not self.hasStatus(STATE_RUNNING):
            print("Cannot complete without running")
            return

        if state == STATE_ERROR:
            print("Error occurred, transitioning to ERROR state")
            self.state = state
            return

        self.state = state
        print(f"Transitioned to state: {bin(self.state)}")

    def getState(self):
        """get state"""
        return bin(self.state)

    def checkInputStatus(self, status):
        """check input status"""
        if status == 1 and self.state == STATE_INIT:
            self.addState(STATE_RUNNING)
            print("model service calculating!")
        elif status == 2:
            self.addState(STATE_COMPLETED)
            print("model calculation was completed!")



"""
Author: DiChen
Date: 2024-09-09 21:19:00
LastEditors: DiChen
LastEditTime: 2024-09-09 21:25:48
"""


class MDL:
    # TODO:应该为动态IP
    def __init__(self):
        self.ip = "172.21.252.204"
        self.port = 8061
        self.origin_lists = {}

    def resolvingMDL(self, mdlData: dict):
        if mdlData:
            self.origin_lists = self.parse_model_data(mdlData)
            return self.origin_lists
        else:
            # TODO: 处理无mdl的情况
            return None

    ####################### private#######################

    def parse_model_data(self, mdl_data: dict):
        def extract_children(udx_node):
            return [
                {
                    "eventId": child["name"],
                    "eventName": child["name"],
                    "eventDesc": child["name"],
                    "eventType": child["type"]
                    .replace("DTKT_", "")
                    .replace("REAL", "FLOAT"),
                    "child": "true",
                    "value": "",
                }
                for child in udx_node.get("UdxNode", [])
            ]

        def process_event(event, evt, dataset_item, data, is_input=True):
            entry_type = "inputs" if is_input else "outputs"
            entry = {
                "statename": event.get("name"),
                "event": evt.get("name"),
                "optional": evt.get("optional"),
            }
            if is_input:
                entry.update(
                    {
                        "url": "",
                        "tag": dataset_item.get("name"),
                        "suffix": "",
                    }
                )
            else:
                entry["template"] = {
                    "type": "id" if "externalId" in dataset_item else "None",
                    "value": dataset_item.get("externalId", ""),
                }

            if dataset_item["type"] == "internal" and dataset_item.get(
                    "UdxDeclaration"
            ):
                udx_node = dataset_item["UdxDeclaration"][0].get("UdxNode")
                if udx_node:
                    entry["children"] = extract_children(
                        dataset_item["UdxDeclaration"][0]["UdxNode"][0]
                    )

            data[entry_type].append(entry)

        data = {
            "outputs": [],
            "port": self.port,  # Fill with actual port if available
            "inputs": [],
            "ip": self.ip,  # Fill with actual IP if available
            "pid": mdl_data.get("md5", ""),
            "oid": mdl_data.get("id", ""),
            "username": "",  # Fill with actual username if available
        }
        related_datasets = mdl_data["mdlJson"]["ModelClass"][0]["Behavior"][0][
            "RelatedDatasets"
        ][0]["DatasetItem"]

        for model_class in mdl_data.get("mdlJson", {}).get("ModelClass", []):
            for behavior in model_class.get("Behavior", []):
                for state_group in behavior.get("StateGroup", []):
                    for state in state_group.get("States", []):
                        for event in state.get("State", []):
                            for evt in event.get("Event", []):
                                dataset_reference = evt.get(
                                    "ResponseParameter",
                                    evt.get("DispatchParameter", []),
                                )
                                for param in dataset_reference:
                                    dataset_item = next(
                                        (
                                            item
                                            for item in related_datasets
                                            if item["name"] == param["datasetReference"]
                                        ),
                                        None,
                                    )
                                    if dataset_item:
                                        process_event(
                                            event,
                                            evt,
                                            dataset_item,
                                            data,
                                            is_input=(evt.get("type")
                                                      == "response"),
                                        )

        return data



"""
Author: DiChen
Date: 2024-09-06 14:24:53
LastEditors: DiChen
LastEditTime: 2024-09-09 18:36:39
"""

################public lib################
import urllib.parse
import time

################private lib################
from .base import Service

from .openUtils import *
from . import constants as C
from . import openModel
__all__ = [
    "urllib",
    "time",
    "Service",
    "openModel",
    "C",
    "MDL",
    "HttpClient",
    "PV",
    "StateManager",
    "NotValueError",
    "modelStatusError",
    "calTimeoutError",
    "UploadFileError",
    "MDLVaildParamsError",
]
